@@ -1,12 +1,14 @@
-# This file is Copyright (c) 2018 Dolu1990 <charles.papon.90@gmail.com>
-# This file is Copyright (c) 2018-2019 Florent Kermarrec <florent@enjoy-digital.fr>
-# This file is Copyright (c) 2018-2019 Sean Cross <sean@xobs.io>
-# This file is Copyright (c) 2019 Tim 'mithro' Ansell <me@mith.ro>
-# This file is Copyright (c) 2019 David Shah <dave@ds0.me>
-# This file is Copyright (c) 2019 Antmicro <www.antmicro.com>
-# This file is Copyright (c) 2019 Kurt Kiefer <kekiefer@gmail.com>
-
-# License: BSD
+#
+# This file is part of LiteX.
+#
+# Copyright (c) 2018 Dolu1990 <charles.papon.90@gmail.com>
+# Copyright (c) 2018-2019 Florent Kermarrec <florent@enjoy-digital.fr>
+# Copyright (c) 2018-2019 Sean Cross <sean@xobs.io>
+# Copyright (c) 2019 Tim 'mithro' Ansell <me@mith.ro>
+# Copyright (c) 2019 David Shah <dave@ds0.me>
+# Copyright (c) 2019 Antmicro <www.antmicro.com>
+# Copyright (c) 2019 Kurt Kiefer <kekiefer@gmail.com>
+# SPDX-License-Identifier: BSD-2-Clause
 
 import os
 
@@ -32,6 +34,8 @@ CPU_VARIANTS = {
     "linux":            "VexRiscv_Linux",
     "linux+debug":      "VexRiscv_LinuxDebug",
     "linux+no-dsp":     "VexRiscv_LinuxNoDspFmax",
+    "secure":           "VexRiscv_Secure",
+    "secure+debug":     "VexRiscv_SecureDebug",
 }
 
 
@@ -56,6 +60,8 @@ GCC_FLAGS = {
     "linux":            "-march=rv32ima    -mabi=ilp32",
     "linux+debug":      "-march=rv32ima    -mabi=ilp32",
     "linux+no-dsp":     "-march=rv32ima    -mabi=ilp32",
+    "secure":           "-march=rv32ima    -mabi=ilp32",
+    "secure+debug":     "-march=rv32ima    -mabi=ilp32",
 }
 
 
